@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# $Id: maxbotix.py 1549 2016-09-22 01:14:05Z mwall $
+# $Id: maxbotix.py 1563 2016-10-25 15:11:37Z mwall $
 # Copyright 2015 Matthew Wall
 
 # From the maxbotix datasheet:
@@ -42,7 +42,8 @@ import weewx.engine
 import weewx.units
 
 DRIVER_NAME = "Maxbotix"
-DRIVER_VERSION = "0.4"
+DRIVER_VERSION = "0.5"
+DEFAULT_MODEL = 'MB7363'
 
 def logmsg(dst, msg):
     syslog.syslog(dst, 'maxbotix: %s' % msg)
@@ -73,20 +74,6 @@ weewx.units.MetricUnits['group_range'] = 'cm'
 weewx.units.MetricWXUnits['group_range'] = 'cm'
 
 
-# no_target value for various types of sensor
-NO_TARGET = {
-    # 5-meter sensors
-    'MB7360': 5000,
-    'MB7369': 5000,
-    'MB7380': 5000,
-    'MB7389': 5000,
-    # 10-meter sensors
-    'MB7363': 9999,
-    'MB7366': 9999,
-    'MB7383': 9999,
-    'MB7386': 9999
-    }
-
 
 class MaxbotixDriver(weewx.drivers.AbstractDevice):
 
@@ -98,10 +85,8 @@ class MaxbotixDriver(weewx.drivers.AbstractDevice):
         loginf("polling interval is %s" % self.poll_interval)
         self.port = stn_dict.get('port', '/dev/ttyUSB0')
         loginf("port is %s" % self.port)
-        model = stn_dict.get('model', 'MB7363')
-        self.no_target = NO_TARGET.get(model, 9999)
-        loginf("using no_target value of %s for sensor type %s" %
-               (self.no_target, model))
+        self.model = stn_dict.get('model', DEFAULT_MODEL)
+        loginf("model is %s" % self.model)
 
     @property
     def hardware_name(self):
@@ -112,17 +97,12 @@ class MaxbotixDriver(weewx.drivers.AbstractDevice):
         while ntries < self.max_tries:
             ntries += 1
             try:
-                with Sensor(self.port) as sensor:
+                with Sensor(self.model, self.port) as sensor:
                     v = sensor.get_range()
                 ntries = 0
-                if v == self.no_target:
-                    loginf("no target detected: v=%s" % v)
-                    v = None
-                else:
-                    v /= 10.0 # sensor is mm, we want cm
                 _packet = {'dateTime': int(time.time() + 0.5),
                            'usUnits': weewx.METRIC,
-                           'range': v}
+                           'range': v / 10.0 if v is not None else None}
                 yield _packet
                 if self.poll_interval:
                     time.sleep(self.poll_interval)
@@ -142,6 +122,7 @@ class MaxbotixService(weewx.engine.StdService):
         loginf("service version is %s" % DRIVER_VERSION)
         self.port = config_dict.get('port', '/dev/ttyUSB0')
         loginf("port is %s" % self.port)
+        self.model = config_dict.get('model', DEFAULT_MODEL)
 
     def handle_new_loop(self, event):
         self.get_data(event.packet)
@@ -152,7 +133,7 @@ class MaxbotixService(weewx.engine.StdService):
     def get_data(self, data):
         v = None
         try:
-            with Sensor(self.port) as sensor:
+            with Sensor(self.model, self.port) as sensor:
                 v = sensor.get_range()
         except (serial.serialutil.SerialException, weewx.WeeWxIOError), e:
             logerr("Failed to get reading: %s" % e)
@@ -165,11 +146,34 @@ class MaxbotixService(weewx.engine.StdService):
 
 class Sensor():
 
-    def __init__(self, port, baudrate=9600, timeout=1):
+    # information about each type of sensor.  the key is the model number.  the
+    # associated tuple contains the units of the value that is returned, the
+    # value the sensor returns when the range is maxxed out, and the number or
+    # characters (excluding the R and trailing newline) in the value string.
+    MODEL_INFO = {
+        'MB1040': ['inch', 254, 3], # 6in min; 254in max; 1in res
+        # 5-meter sensors
+        'MB7360': ['mm', 5000, 4],
+        'MB7369': ['mm', 5000, 4],
+        'MB7380': ['mm', 5000, 4],
+        'MB7389': ['mm', 5000, 4],
+        # 10-meter sensors
+        'MB7363': ['mm', 9999, 4],
+        'MB7366': ['mm', 9999, 4],
+        'MB7383': ['mm', 9999, 4],
+        'MB7386': ['mm', 9999, 4]
+        }
+
+    def __init__(self, model, port, baudrate=9600, timeout=1):
+        self.model = model
         self.port = port
         self.baudrate = baudrate
         self.timeout = timeout
         self.serial_port = None
+        model_info = Sensor.MODEL_INFO[self.model]
+        self.units = model_info[0]
+        self.no_target = model_info[1]
+        self.data_length = model_info[2]
 
     def __enter__(self):
         self.open()
@@ -188,12 +192,18 @@ class Sensor():
             self.serial_port = None
 
     def get_range(self):
-        line = self.serial_port.read(6)
+        # return value is always mm
+        line = self.serial_port.read(self.data_length + 2)
         if line:
             line = line.strip()
-        if line and len(line) == 5 and line[0] == 'R':
+        if line and len(line) == self.data_length + 1 and line[0] == 'R':
             try:
                 v = int(line[1:])
+                if v == self.no_target:
+                    logdbg("no target detected: v=%s" % v)
+                    v = None
+                if self.units == 'inch':
+                    v *= 25.4
                 return v
             except ValueError, e:
                 raise weewx.WeeWxIOError("bogus value: %s" % e)
@@ -213,6 +223,9 @@ if __name__ == "__main__":
         parser.add_option('--port', dest="port", metavar="PORT",
                           default='/dev/ttyUSB0',
                           help="The port to use. Default is '/dev/ttyUSB0'.")
+        parser.add_option('--model', dest="model", metavar="MODEL",
+                          default=DEFAULT_MODEL,
+                          help="The sensor model. Default is %s" % DEFAULT_MODEL)
         parser.add_option('--test-sensor', dest='tc', action='store_true',
                           help='test the sensor')
         parser.add_option('--test-driver', dest='td', action='store_true',
@@ -222,11 +235,11 @@ if __name__ == "__main__":
         (options, args) = parser.parse_args()
 
         if options.tc:
-            test_sensor(options.port)
+            test_sensor(options.model, options.port)
         elif options.td:
             test_driver()
         elif options.ts:
-            test_service(options.port)
+            test_service(options.model, options.port)
 
     def test_driver():
         import weeutil.weeutil
@@ -235,7 +248,7 @@ if __name__ == "__main__":
         for pkt in driver.genLoopPackets():
             print weeutil.weeutil.timestamp_to_string(pkt['dateTime']), pkt
 
-    def test_service(port):
+    def test_service(model, port):
         import sys
         config = {
             'Station': {
@@ -247,6 +260,7 @@ if __name__ == "__main__":
                 'driver': 'weewx.drivers.simulator',
                 'mode': 'simulator'},
             'Maxbotix': {
+                'model': model,
                 'port': port},
             'Engine': {
                 'Services': {
@@ -261,10 +275,10 @@ if __name__ == "__main__":
             sys.stdout.flush()
             time.sleep(1)
 
-    def test_sensor(port):
+    def test_sensor(model, port):
         import sys
         print "range is mm"
-        with Sensor(port) as sensor:
+        with Sensor(model, port) as sensor:
             while True:
                 sys.stdout.write("\r%s" % sensor.get_range())
                 sys.stdout.flush()
